/* atmxram.h */

/* This header file defines the layout of XRAM */

/* Copyright (c) 1996  Robert Geist and James Westall
 *                     Clemson University Dept. of Computer Science
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#define XRAM_SIZE      (16 * 1024)

/* BEWARE of how you change these values.. Some crucial alignment */
/* constraints are enforced by the APE 25                         */

#define LCI_BITS          7
#define LC_COUNT         64        /* Max # of LC's          */
#define LCI_COUNT       128
#define TCB_COUNT         2        /* # of xmit cell buffers */
#define R1CB_COUNT       16        /* Various receive cell   */
#define R5CB_COUNT       72        /* ... buffers            */
#define ROCB_COUNT       15

#define T1RQ_COUNT       16
#define T5RQ_COUNT       24

#define VP_TABLE_BASE    0
#define VP_TABLE_SIZE    512

#define VCI_MASK         LCI_COUNT-1

#define LCI_TABLE_BASE  (VP_TABLE_BASE + VP_TABLE_SIZE)
#define LCI_TABLE_SIZE  (2 * LCI_COUNT)

#define TCB_TABLE_BASE  (LCI_TABLE_BASE + LCI_TABLE_SIZE)
#define TCB_TABLE_SIZE  (64 * TCB_COUNT)

#define R1CB_TABLE_BASE (TCB_TABLE_BASE + TCB_TABLE_SIZE)
#define R1CB_TABLE_SIZE (64 * R1CB_COUNT)

#define R5CB_TABLE_BASE (R1CB_TABLE_BASE + R1CB_TABLE_SIZE)
#define R5CB_TABLE_SIZE (64 * R5CB_COUNT)

#define ROCB_TABLE_BASE (R5CB_TABLE_BASE + R5CB_TABLE_SIZE)
#define ROCB_TABLE_SIZE (64 * ROCB_COUNT)

#define T1RQ_TABLE_BASE (ROCB_TABLE_BASE + ROCB_TABLE_SIZE)
#define T1RQ_TABLE_SIZE (8 * T1RQ_COUNT)

#define T5RQ_TABLE_BASE (T1RQ_TABLE_BASE + T1RQ_TABLE_SIZE)
#define T5RQ_TABLE_SIZE (8 * T5RQ_COUNT)


#define LC_TABLE_BASE   (T5RQ_TABLE_BASE + T5RQ_TABLE_SIZE)
#define LC_TABLE_SIZE   (128 * LC_COUNT)
#define LC_TABLE_END    (LC_TABLE_BASE + LC_TABLE_SIZE)

#define FIRST_LCID      (LC_TABLE_BASE >> 7)

/* With the above defines... xram is mapped as follows:

VP_TABLE_BASE   - 00000000
VP_TABLE_SIZE   - 00000200
LCI_TABLE_BASE  - 00000200
LCI_TABLE_SIZE  - 00000080
TCB_TABLE_BASE  - 00000280
TCB_TABLE_SIZE  - 00000080
R1CB_TABLE_BASE - 00000300
R1CB_TABLE_SIZE - 00000400
R5CB_TABLE_BASE - 00000700
R5CB_TABLE_SIZE - 00001200
ROCB_TABLE_BASE - 00001900
ROCB_TABLE_SIZE - 000003c0
T1RQ_TABLE_BASE - 00001cc0
T1RQ_TABLE_SIZE - 00000080
T5RQ_TABLE_BASE - 00001d40
T5RQ_TABLE_SIZE - 000000c0
LC_TABLE_BASE   - 00001e00
LC_TABLE_SIZE   - 00002000
LC_TABLE_END    - 00003e00

  which is within the constraints of the 16K available */


